@@ -1,5 +1,5 @@
 cv.hsvmpath <- function(outlist, lambda, x, y, foldid, 
-    pred.loss, delta) {
+    pred.loss, delta, omega) {
     typenames <- c(misclass = "Misclassification Error", loss = "Margin Based Loss")
     if (pred.loss == "default") 
         pred.loss <- "loss"
@@ -23,9 +23,7 @@ cv.hsvmpath <- function(outlist, lambda, x, y, foldid,
     }
     cvraw <- switch(pred.loss, loss = 2 * hubercls(y * predmat, 
         delta), misclass = (y != ifelse(predmat > 0, 1, -1)))
-    cvob <- cvcompute(cvraw, foldid, nlams)
-    cvraw <- cvob$cvraw
-    N <- cvob$N
+    N <- length(y) - apply(is.na(predmat), 2, sum)
     cvm <- apply(cvraw, 2, mean, na.rm = TRUE)
     cvsd <- sqrt(apply(scale(cvraw, cvm, FALSE)^2, 2, mean, na.rm = TRUE)/(N - 
         1))
